"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request-promise");
var moment = require("moment");
var WrapperLogger = /** @class */ (function () {
    function WrapperLogger(logger) {
        this.logger = logger || {};
    }
    WrapperLogger.prototype.info = function () {
        this.logger.info && this.logger.info.apply(this.logger, arguments);
    };
    WrapperLogger.prototype.error = function () {
        this.logger.error && this.logger.error.apply(this.logger, arguments);
    };
    WrapperLogger.prototype.debug = function () {
        this.logger.debug && this.logger.debug.apply(this.logger, arguments);
    };
    return WrapperLogger;
}());
var HttpClient = /** @class */ (function () {
    function HttpClient(baseUrl, logger, basicAuthUser, basicAuthPwd, bearer) {
        if (!baseUrl)
            throw new Error("baseUrl must be defined");
        this._baseUrl = baseUrl;
        this._logger = new WrapperLogger(logger);
        if (basicAuthUser && basicAuthPwd) {
            this._defaultHeaders = {
                "Authorization": "Basic " + new Buffer(basicAuthUser + ":" + basicAuthPwd).toString("base64")
            };
        }
        else if (bearer) {
            this._defaultHeaders = {
                "Authorization": "Bearer " + bearer
            };
        }
    }
    HttpClient.prototype._request = function (method, path, body) {
        return __awaiter(this, void 0, void 0, function () {
            var url, start, options, result, error_1, message;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        url = "" + this._baseUrl + path;
                        start = moment();
                        options = {
                            method: method,
                            uri: url,
                            body: body,
                            headers: this._defaultHeaders,
                            resolveWithFullResponse: true,
                            json: true
                        };
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, request(options)];
                    case 2:
                        result = _a.sent();
                        this._logger.info(method + " " + url + " " + result.statusCode + " (" + moment().diff(start, "milliseconds") + " ms)");
                        this._logger.debug("request-options", options);
                        return [2 /*return*/, result.body];
                    case 3:
                        error_1 = _a.sent();
                        message = "";
                        if (error_1.response && error_1.response.errors && error_1.response.errors.length > 0) {
                            message = error_1.response.errors.join(",");
                        }
                        else {
                            message = error_1.toString && error_1.toString();
                        }
                        this._logger.error("\n"
                            + "--------------------------------------------------------------------\n"
                            + (method + " " + url + " " + (error_1.response && error_1.response.statusCode || "<unknown statuscode>") + " (" + moment().diff(start, "milliseconds") + " ms)\n")
                            + ("request-options: " + JSON.stringify(options) + "\n")
                            + ("error:" + message + "\n") + "--------------------------------------------------------------------");
                        throw error_1;
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    HttpClient.prototype.get = function (route) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._request("GET", route)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    HttpClient.prototype.post = function (route, payload) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._request("POST", route, payload)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    HttpClient.prototype.patch = function (route, payload) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._request("PATCH", route, payload)];
                    case 1: return [2 /*return*/, (_a.sent())];
                }
            });
        });
    };
    HttpClient.prototype.put = function (route, payload) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._request("PUT", route, payload)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    HttpClient.prototype.delete = function (route) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._request("DELETE", route)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return HttpClient;
}());
exports.HttpClient = HttpClient;
var TestHttpClient = /** @class */ (function () {
    function TestHttpClient(routePayloads) {
        this._routePayloads = routePayloads;
        this.calls = { get: {}, post: {}, patch: {}, delete: {} };
    }
    TestHttpClient.prototype.get = function (route) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                this.calls.get[route] = {};
                result = this._routePayloads.get[route];
                return [2 /*return*/, this._routePayloads.get[route]];
            });
        });
    };
    TestHttpClient.prototype.post = function (route, payload) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.calls.post[route] = payload || {};
                return [2 /*return*/, this._routePayloads.post[route]];
            });
        });
    };
    TestHttpClient.prototype.patch = function (route, payload) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.calls.patch[route] = payload || {};
                return [2 /*return*/, this._routePayloads.patch[route]];
            });
        });
    };
    TestHttpClient.prototype.delete = function (route) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.calls.delete[route] = undefined;
                return [2 /*return*/, this._routePayloads.delete[route]];
            });
        });
    };
    return TestHttpClient;
}());
exports.TestHttpClient = TestHttpClient;
//# sourceMappingURL=index.js.map